const path = require("path");
const { CheckerPlugin } = require("awesome-typescript-loader");
const { TsConfigPathsPlugin } = require("awesome-typescript-loader");
const CopyWebpackPlugin = require("copy-webpack-plugin");
const CleanWebpackPlugin = require("clean-webpack-plugin");
const HtmlWebpackPlugin = require("html-webpack-plugin");
const UglifyJSPlugin = require("uglifyjs-webpack-plugin");

module.exports = {
    entry: "./src/ts/bootstrap.ts",
    output: {
        filename: "dist/app.bundle.js",
        path: path.resolve(__dirname, ".")
    },
    node: {
        fs: "empty",
        module: "empty"
    },
    resolve: {
        extensions: [
            ".ts",
            ".js"
        ]
    },
    devtool: "source-map",
    module: {
        rules: [
            {
                test: /\.ts$/,
                loader: "awesome-typescript-loader"
            }
        ]
    },
    plugins: [
        new CleanWebpackPlugin(
            [
                "dist"
            ],
            {
                verbose: false,
                exclude: [
                    ".gitkeep"
                ]
            }
        ),
        new CheckerPlugin(),
        new TsConfigPathsPlugin(),
        new CopyWebpackPlugin(
            [
                {
                    context: "./src",
                    from: "**/*",
                    to: "./dist/",
                    ignore: [
                        "ts/**/*",
                        ".gitkeep",
                        "index.html"
                    ]
                }
            ]
        ),
        new UglifyJSPlugin({
            compress: {
                drop_debugger: false,
                drop_console: false,
                sequences: true,
                properties: true,
                dead_code: true,
                unsafe: true,
                unsafe_comps: true,
                unsafe_math: true,
                unsafe_proto: true,
                unsafe_regexp: true,
                conditionals: true,
                comparisons: true,
                evaluate: true,
                booleans: true,
                loops: true,
                unused: true,
                toplevel: true,
                top_retain: [],
                hoist_funs: true,
                hoist_vars: true,
                if_return: true,
                join_vars: true,
                cascade: true,
                collapse_vars: true,
                reduce_vars: true,
                warnings: false,
                negate_iife: true,
                pure_getters: "strict",
                pure_funcs: [],
                expression: false,
                keep_fargs: true,
                keep_fnames: false,
                passes: 3,
                keep_infinity: false,
                side_effects: true
            },
            mangle: {
                toplevel: true,
                eval: true,
                reserved: [],
                keep_fnames: true
            },
            comments: false,
            sourceMap: true
        }),
        new HtmlWebpackPlugin({
            filename: "dist/index.html",
            template: "src/index.html",
            inject: "body",
            minify: {
                caseSensitive: false,
                collapseBooleanAttributes: true,
                collapseInlineTagWhitespace: true,
                collapseWhitespace: true,
                conservativeCollapse: false,
                decodeEntities: true,
                html5: true,
                includeAutoGeneratedTags: true,
                keepClosingSlash: false,
                minifyCSS: {
                    compatibility: "*",
                    format: false,
                    inline: [
                        'local'
                    ],
                    level: 2,
                    rebase: true,
                    rebaseTo: "dist",
                    sourceMap: true
                },
                minifyJS: false, // Doesn't work for some odd reason, so we will use another plugin for that
                minifyURLs: {
                    removeEmptyDirectoryNames: true
                },
                preserveLineBreaks: false,
                preventAttributesEscaping: false,
                processConditionalComments: false,
                removeAttributeQuotes: false,
                removeComments: true,
                removeEmptyAttributes: true,
                removeRedundantAttributes: true,
                removeScriptTypeAttributes: true,
                removeStyleLinkTypeAttributes: true,
                sortAttributes: true,
                sortClassName: true,
                trimCustomFragments: false,
                useShortDoctype: true
            }
        })
    ]
};